; Temperaturmessprogramm Version 7
; mit AD-Wandlerroutine und Speichern der Messwerte ins EEPROM
;
; Auslesen mit Terminalprogramm des Lernpaket "Microconotroller"
;
; Messung mit NTC-Widerstand 1kOhm
; und Spannungsteiler 1,5kOhm
; an Port 4
;
; Messroutine misst jede stunde 1 Wert
; Beginn: 15-6-2008 Felix Herzog
; Fertig: 08-7-2008 neues Timing
; www.wherzog.de
;
;; 1,2 MHZ Taktfrequenz des TINY13

;; Header fr TINY13
.include "tn13def.inc"

;;Port B Pin Definitionen fr RS-232 Routine
.equ   TXD    = 1
.equ   RXD    = 2

;; Variable fr Messdauer
.equ   messinterval = 60		;eine Messung alle 60 Minuten

;; Variablendefinitionen
								;AD Wandlerdaten
.def A = r10					;A hat High-Byte
.def B = r11					;B hat Low-Byte
.def chanel = r15				;AD Wandler Kanal

.def temp = r16					;Temporre Variable
.def leds = r17
.def   Delay  = r19				;r18 r19 Doppelt vergeben weil z1 register bei Ausgabe nicht in Gebrauch
.def   Count  = r18				;evtl. noch Aufrumen

.def z1 = r18					;Zhler in Interrupt-Routine
.def z2 = r19
.def z3 = r20

.def   EEadr  = r21				;Adresse EEPROM
.def   EEmode = r22				;EEPROM Schreibmodus

.def sicher = r23				;zum SREG sichern


;; Interrupt Vektorentabelle 
        rjmp    main            ;Reset Handler

.org OVF0addr
        rjmp    timer0_overflow ;Timer Overflow Handler fr Interrupt setzen
 
;; *** Beginn des Hauptprogramm ***
main:

;; Ein- Ausgabepin initialisieren
        ldi     temp, 0x10      ;Port B Pin 4 auf Ausgang weil LED anschlossen
        out     DDRB, temp		;alle anderen Pins sind Eingang

;; LED Einschalten
		ldi leds,0				;LED Flag initialisieren 
	   	sbi portb,4				;LED Einschalten

;; Jetzt Statustaster abfragen an PB0
	    sbis pinb,0				;berspringen wenn PB0=1 - also Mode-Schalter offen
		rjmp messaus			;Serielle Datenausgabe aufrufen statt Messung

;; AD Wandler initialisieren
AdcInit:
      ldi   temp,3     			;Clock / 4 Abfragefrequenz des AD Wandlers
      out   ADCSRA,temp	
      sbi   ADCSRA,ADEN 		;AD Wandler einschalten 

	  ldi   temp, 3				; Kanal 3 des AD Wandlers (= PB3) in temp
	  mov   chanel, temp		; weil r15 (channel) nur mit Register bedient werden kann

;; Timer 0 initialisieren
;		ldi temp,(1<<CS00)		; Zum Testen ohne Prescaler, dann nachfolgende Zeile auskommentieren
	    ldi temp,(1<<CS02 | 1<<CS00) ; Clock Select0 und 2 setzen -> Vorteiler 1024
	    out TCCR0B,temp         ; Timer Counter Control Register
	   
	    ldi temp,(1<<TOIE0) 	; Timer/Counter overflow Interrupt enable bit
	    out TIMSK0,temp	    	; ins Timer/Counter Interrupt Register schreiben
	   
;; Timer Zhlschleifenregister initialisiern
		ldi	z1, 0x00
		ldi z2, 0x00
		ldi z3, 0x00

;; Jetzt beginnt die Messung im Interrupt!
		rcall tempmess			; eine Testmessnung damit Wandler korrekt initialisiert ist
		clr EEadr				; Zhler fr EEPROM Adresse = 0
	    sei						; Global alle Interrupts freigeben - Messung luft

;; Hauptprogrammschleife im Mess-Modus
loop:							; Schleife fr Mess-Interrupt!
		nop   					; Nix tun, ist alles im Interrupt drinne
		rjmp loop
		
;; ENDE Hauptprogrammschleife

;-----------------------------------------------
;; Ausgabe der Messwerte im EEPROM per RS-232
;; Dazu PB0 mit GND verbinden und 
;; durch ein RTS Impuls im Terminalprogramm aktivieren...
;---------------------------------------
messaus:
		cli						; Interrupt sperren
Anfang:
        sbi    ddrb,TXD			;Datenrichtung TXD - also PIN PB2 als Ausgang schalten
		ldi    EEadr,0			;Adresszhler auf Null

Schleife1:
        rcall  RdEE
		rcall  LEDschalt		;LED umschalten zur Aktivittsanzeige
		rcall  WrCOM
		inc    EEadr
		cpi    EEadr,64			;Alle Werte gesendet ?
		brlo   Schleife1

		cbi		portb,4  		;LED aus	
mausfertig:
		nop
		rjmp   mausfertig
;; ENDE der Messdatenausgabe


;--------------------------------------------
;; Interrupt-Routine 
;--------------------------------------------
timer0_overflow:                    ; Timer 0 Overflow Handler
		in 		sicher,SREG			; Statusregsiter sichern		
		cpi		EEAdr,64			; Haben wir 64 Bytes = 32 Messungen gemacht?
		brsh 	messfertig			; Ja, also Messung berspringen

;; Zhlschleife zum Verzgern der Messungen (hier ca. alle 60 Min eine Messung)
	    inc		z1					; Zhler1 erhhen
		cpi		z1,67				; Maximum? (67,24)
		breq	timerz2 			; Ja, dann weiter mit Zhler 2
		rjmp	fertig				; sonst Interrupt Ende
timerz2:
		rcall   LEDschalt			; LED umschalten zum Anzeige der Messung
	    inc		z2					; Zhler2 erhhen
		cpi		z2,messinterval				; 60 minuten fr ca. 1h Messinterval
		breq	timermess 			; Ja, dann weiter mit Messung
		rjmp	fertig

;; Messung durchfhren 
timermess:				
		rcall	tempmess			; AD Wandler auslesen
		rcall   WrEE				; 2 Byte ins EPROM schreiben und EEAdr erhhen
		rcall   LEDschalt			; jetzt wird LED umgeschaltet

;; Messung ist fertig
messfertig:
		cbi portb,4					; Status-LED ausschalten (oder besser einschalten?)
		ldi	z1, 0x00
		ldi z2, 0x00
		ldi z3, 0x00

fertig:
		out SREG,sicher				;Statusregister wiederherstellen
        reti

;-----------------------------------
;; Unterprogramm zum LED umschalten
;; .def leds speichert LED Status 
;-----------------------------------
LEDschalt:
		cpi		leds, 0				;leds ist Null?
		breq	LEDeinschalt		;ja, also einschalten
		sbi		portb,4				;sonst LED ausschalten
		com     leds				;leds invertieren
		rjmp 	LEDfertig

LEDeinschalt:
		com     leds				;leds invertieren
		cbi     portb,4				;LED anschalten
LEDfertig:
		reti

;----------------------------------------
;; Unterprogramm AD Wandler abfragen 
;; Hier zur Temperaturmessung mit NTC 
;----------------------------------------
tempmess:

RdADC:
      out   ADMUX,chanel		;In chanel ist die Nr des Kanals
      sbi   ADCSRA,ADSC   		;Wandlung starten
ADrdy:
      sbic  ADCSRA,ADSC   
      rjmp  ADrdy
      sbi   ADCSRA,ADSC
ADrdyb:
      sbic  ADCSRA,ADSC   
      rjmp  ADrdyb
      in    B,ADCL				; B hat Low-Byte
      in    A,ADCH				; A hat High-Byte
      ret

;----------------------------------
;Daten ins EEPROM schreiben
;    .def   EEadr  = EEPROM Adresse
;    .def   EEmode = Schreibmodus
;	 .def   A und B = Schreibdaten
;----------------------------------
WrEE:   sbic  EECR,EEWE         ;Warten bis Schreibbit EEWE Null wird
        rjmp  WrEE
        ldi   EEmode,0
        out   EECR,EEmode	    ;EEPROM control regsiter auf Null
        out   EEARL,EEadr		;EEPROM Adress Register LOW mit Adresse 
        out   EEDR,A			;Schreibdaten in A
        sbi   EECR,EEMPE	    ;EEPROM Master Program enable
        sbi   EECR,EEPE			;EEPROM Program Enable setzen

		inc	  EEadr				;EEPROM Adressbyte eins hochzhlen
WrEE2:
		sbic  EECR,EEWE         ;Warten bis Schreibbit EEWE Null wird
        rjmp  WrEE2
        ldi   EEmode,0
        out   EECR,EEmode	    ;EEPROM control regsiter auf Null
        out   EEARL,EEadr		;EEPROM Adress Register LOW mit Adresse 
        out   EEDR,B			;Schreibdaten in B
        sbi   EECR,EEMPE	    ;EEPROM Master Program enable
        sbi   EECR,EEPE			;EEPROM Program Enable setzen
        
		inc	  EEadr				;EEPROM Adressbyte eins hochzhlen
        ret						;Rcksprung

;----------------------------------
;Daten aus EEPROM lesen
;    .def   EEadr  = EEPROM Adresse
;    .def   EEmode = Schreibmodus
;	 .def   A = gelesene Daten
;-----------------------------------

RdEE:   sbic  EECR,EEWE	
        rjmp  RdEE
        out   EEAR,EEadr
        sbi   EECR,EERE
        in    A,EEDR
        ret

;-----------------------------------
; Ein Byte ber RS-232 ausgeben
; Byte ist in A 
;-----------------------------------

WrCOM:  sbi   portb,TXD  		;Senden
        ldi   Delay,38 
D4:     dec   Delay
        brne  D4
        ldi   Count,8
L2:     sbrc  A,0
        rjmp  OFF
        rjmp  ON
ON:     sbi   portb,TXD
        rjmp  BitD
OFF:    cbi   portb,TXD
        rjmp  BitD
BitD:   ldi   Delay,38  
D5:     dec   Delay
        brne  D5
        lsr   A
        dec   Count
        brne  L2
        cbi   PORTB,TXD
        ldi   Delay,38  
D6:     dec   Delay
        brne  D6
        ret

